function Game() {
  
  this.WIDTH = 960;
  this.HEIGHT = 600;
  
  this.DEBUG = false;
  
  this.TIME_PER_FRAME = 20;
  
  this.interval;
  
  this.state = -1;
  this.nextState = -1;
  
  this.loadingScreen;
  this.loginScreen;
  
  
  this.init = function() {
    this.drawEmpty(); 
    timer.init();
    physics.init();
    performanceMonitor.init();
    preloadingManager.preloadLoadingScreen();
  };
  
  
  this.showLoadingScreen = function() {
    this.setState(Game.LOADING_SCREEN_STATE);
    this.startLoop();
  };
  
  
  this.setState = function(state) {
    this.nextState = state;
  };
  
  
  this.startLoop = function() {
    this.interval = window.setInterval(function() {
      game.loop();
    }, this.TIME_PER_FRAME);
  };
  
  
  this.loop = function() {
    this.initState();
    timer.update();
    this.update();
    this.draw();
    performanceMonitor.update();
  };
  
  
  this.initState = function() {
    if(this.state != this.nextState) {
      this.state = this.nextState;
      
      if(this.state == Game.LOADING_SCREEN_STATE) {
        this.loadingScreen = new LoadingScreen();
        preloadingManager.preloadGame();
        
      } else if(this.state == Game.LOGIN_SCREEN_STATE) {
        this.loginScreen = new LoginScreen();
        this.loginScreen.init();
        
      } else if(this.state == Game.INGAME_STATE) {
        this.setupGame();
        
      }
    }
  };
  
  
  this.setupGame = function() {
    
    keyboard.init();
    mouse.init();
    
    physics.setup();
    
    map = new Map();
    map.init();
    
    player = new Player();
    player.init();
    
    quest = new Quest();
    quest.init();
    
    panel = new Panel();
    panel.init();
    
    upgradeWindow = new UpgradeWindow();
    upgradeWindow.init();
    
    keyboard.registerKeyUpHandler(Keyboard.P, function() {
      if(panel.window == null) {
        game.paused = !game.paused;
        panel.updatePauseMuteButtons();
      }
    });
    
    keyboard.registerKeyUpHandler(Keyboard.M, function() {
      sound.muteUnmute();
      panel.updatePauseMuteButtons();
    });
    
  };
  
  
  this.update = function() {
    
    if(this.state == Game.LOADING_SCREEN_STATE) {
      this.loadingScreen.update();

    } else if(this.state == Game.INGAME_STATE) {
      
      if(!game.paused) {
        player.update();
        map.update();
        physics.update();
        quest.update();
        player.applyPhysics();
        map.applyPhysics();
      }
      player.updateMoney();
      
    } 
  };
  
  
  this.draw = function() {
    
    this.drawEmpty();
    
    if(this.state == Game.LOADING_SCREEN_STATE) {
      this.loadingScreen.draw();
      
    } else if(this.state == Game.LOGIN_SCREEN_STATE) {
      this.loginScreen.draw();
      
    } else if(this.state == Game.INGAME_STATE) {
      
      c.fillStyle = "#000";
      c.fillRect(0, 0, this.WIDTH, this.HEIGHT);

      player.applyCamera();
      map.drawStars();
      player.drawTrace();
      map.draw();
      player.draw();
      map.drawNPCsParticles();
      quest.draw();
      player.removeCamera();
      map.drawNPCsLife();
      quest.drawQuestArrow();
      map.drawPlanetInfo();
      
      panel.draw();
      
      if(false) { // DEBUG
        map.drawDebug();
        player.drawDebug();
      }
    }
  };
  
  
  this.drawEmpty = function() {
    c.fillStyle = "#000";
    c.fillRect(0, 0, this.WIDTH, this.HEIGHT);
  };

}


Game.LOADING_SCREEN_STATE = 0;
Game.LOGIN_SCREEN_STATE = 1;
Game.INGAME_STATE = 2;