function Map() {
  
    this.BLOCK_SIZE = 2048;
    this.SIZE = 8192; // has to be multiple of block size
    this.PLANET_DENSITY = 16; // has to be integer
    this.MIN_PLANET_DISTANCE = 200;
    this.CHANCE_OF_MOON = 3;
    
    this.NUM_OF_NPCS = 150;
    
    this.STAR_TEXTURE_WIDTH = 600;
  
    this.planets = [];
    this.npcs = [];
    this.numOfPlanets = 0;
  
    
    this.init = function() {
      this.generateUniverse();
      
      for(var i = 0; i < this.NUM_OF_NPCS; i++) {
        this.npcs[i] = new NPC();
        this.npcs[i].init(i);
      }
    };
    
    
    this.generateUniverse = function() {
      
      var numOfBlocks = (this.SIZE * this.SIZE) / (this.BLOCK_SIZE * this.BLOCK_SIZE);
      
      var planetChoice = [];
      
      var numOfPlanetTypes = Planet.types.length;
      for(var i = 0; i < numOfBlocks * this.PLANET_DENSITY; i++) {
        
        var planetType = Planet.types[i % numOfPlanetTypes];
        var questLevel = planetType.questLevel;
        var hasMoon = false;
        var moonHasQuest = false;
          
        var batch = Math.floor(i / numOfPlanetTypes) % 6;
        if(batch == 3 || batch == 4 || batch == 5) {
          var questLevel = -1;
        }
        if(batch == 4 || batch == 5) {
          var hasMoon = true;
        }
        if(batch == 4) {
          var moonHasQuest = true;
        }
        planetChoice[i] = new Planet();
        planetChoice[i].init(planetType.type, questLevel, planetType.radius, hasMoon, moonHasQuest);
      }

      planetChoice = shuffle(planetChoice);
      
      for(var block = 0; block < numOfBlocks; block++) {
        var blockX = block % Math.floor(this.SIZE / this.BLOCK_SIZE);
        var blockY = Math.floor(block / Math.floor(this.SIZE / this.BLOCK_SIZE));
        
        for(var i = 0; i < this.PLANET_DENSITY; i++) {
          var index = (block * this.PLANET_DENSITY) + i;
          var foundSpot = false;
          var x;
          var y;
          var radius = planetChoice[index].radius;
          var blockPadding = Math.ceil(this.MIN_PLANET_DISTANCE / 2) + radius;
          while(!foundSpot) {            
            x = (blockX * this.BLOCK_SIZE);
            x += rand(blockPadding, this.BLOCK_SIZE - blockPadding);
            y = (blockY * this.BLOCK_SIZE);
            y += rand(blockPadding, this.BLOCK_SIZE - blockPadding);
            foundSpot = true;
            for(var j = block * this.PLANET_DENSITY; j < index; j++) {
              var otherX = this.planets[j].x;
              var otherY = this.planets[j].y;
              var deltaX = x - otherX;
              var deltaY = y - otherY;
              var dist = Math.sqrt((deltaX * deltaX) + (deltaY * deltaY));
              var minDist = radius + this.MIN_PLANET_DISTANCE + this.planets[j].radius;
              if(dist < minDist) {
                foundSpot = false;
              }
            }
          }
          
          this.planets[index] = planetChoice[index];
          this.planets[index].place(index, x, y, -1);
        }
      }
      
      var numOfMoonTypes = Planet.moonTypes.length;
      for(var i = 0; i < numOfBlocks * this.PLANET_DENSITY; i++) {
        if(this.planets[i].hasMoon) {
          var index = this.planets.length;
          var moonType = Planet.moonTypes[i % numOfMoonTypes];
          var questLevel = moonType.questLevel;
          if(!this.planets[i].moonHasQuest) {
            questLevel = -1;
          }
          this.planets[index] = new Planet();
          this.planets[index].init(moonType.type, questLevel, moonType.radius, false, false);
          this.planets[index].place(index, 0, 0, i);
        }
      }
      
      this.numOfPlanets = this.planets.length;
      
    };
    
    
    this.update = function() {
      for(var i = 0; i < this.numOfPlanets; i++) {
        this.planets[i].update();
      }
      for(var i = 0; i < this.NUM_OF_NPCS; i++) {
        this.npcs[i].update();
      }
    };
    
    
    this.applyPhysics = function() {
      for(var i = 0; i < this.NUM_OF_NPCS; i++) {
        this.npcs[i].applyPhysics();
      }
    };
    
    
    this.drawStars = function() {
      var startX = Math.floor((player.x - 2 * this.STAR_TEXTURE_WIDTH) / this.STAR_TEXTURE_WIDTH);
      var startY = Math.floor((player.y - 2 * this.STAR_TEXTURE_WIDTH) / this.STAR_TEXTURE_WIDTH);
      
      var offsetX = (player.x * 0.9) - (this.STAR_TEXTURE_WIDTH * Math.floor((player.x * 0.9) / this.STAR_TEXTURE_WIDTH));
      var offsetY = (player.y * 0.9) - (this.STAR_TEXTURE_WIDTH * Math.floor((player.y * 0.9) / this.STAR_TEXTURE_WIDTH));
      for(var x = 0; x < 4; x++) {
        for(var y = 0; y < 4; y++) {
          var imgX = ((startX + x) * this.STAR_TEXTURE_WIDTH) + offsetX;
          var imgY = ((startY + y) * this.STAR_TEXTURE_WIDTH) + offsetY;
          img.draw("stars0", imgX, imgY);
        }
      }
      
      offsetX = (player.x * 0.7) - (this.STAR_TEXTURE_WIDTH * Math.floor((player.x * 0.7) / this.STAR_TEXTURE_WIDTH));
      offsetY = (player.y * 0.7) - (this.STAR_TEXTURE_WIDTH * Math.floor((player.y * 0.7) / this.STAR_TEXTURE_WIDTH));
      for(var x = 0; x < 4; x++) {
        for(var y = 0; y < 4; y++) {
          var imgX = ((startX + x) * this.STAR_TEXTURE_WIDTH) + offsetX;
          var imgY = ((startY + y) * this.STAR_TEXTURE_WIDTH) + offsetY;
          img.draw("stars1", imgX, imgY);
        }
      }
      
      offsetX = (player.x * 0.3) - (this.STAR_TEXTURE_WIDTH * Math.floor((player.x * 0.3) / this.STAR_TEXTURE_WIDTH));
      offsetY = (player.y * 0.3) - (this.STAR_TEXTURE_WIDTH * Math.floor((player.y * 0.3) / this.STAR_TEXTURE_WIDTH));
      for(var x = 0; x < 4; x++) {
        for(var y = 0; y < 4; y++) {
          var imgX = ((startX + x) * this.STAR_TEXTURE_WIDTH) + offsetX;
          var imgY = ((startY + y) * this.STAR_TEXTURE_WIDTH) + offsetY;
          img.draw("stars2", imgX, imgY);
        }
      }
    };
    
    
    this.draw = function() {
      for(var i = 0; i < this.numOfPlanets; i++) {
        this.planets[i].draw();
      }
      for(var i = 0; i < this.NUM_OF_NPCS; i++) {
        this.npcs[i].draw();
      }
    };
    
    
    this.drawNPCsParticles = function() {
      for(var i = 0; i < this.NUM_OF_NPCS; i++) {
        this.npcs[i].drawParticles();
      }
    };
    
    
    this.drawPlanetInfo = function() {
      for(var i = 0; i < this.numOfPlanets; i++) {
        this.planets[i].drawInfo();
      }
    };
    
    
    this.drawNPCsLife = function() {
      for(var i = 0; i < this.NUM_OF_NPCS; i++) {
        this.npcs[i].drawLife();
      }
    };
    
    
    this.drawDebug = function() {
      for(var i = 0; i < this.numOfPlanets; i++) {
        this.planets[i].drawDebug();
      }
      for(var i = 0; i < this.NUM_OF_NPCS; i++) {
        this.npcs[i].drawDebug();
      }
    };
  
}