function NPC() {
  
  this.DISTANCE_TO_PLANET = 50;
  this.DESTINATION_RADIUS = 30;
  
  this.FORCE = 0.1;
  this.LINEAR_DAMPING = 0.8;
  this.ANGULAR_DAMPING = 1.2;
  this.ROTATION_SPEED = 1.5;
  this.FAKE_SPEED = 60;
  this.GIVE_XP = 20; //10;
  
  this.index;
  
  this.width = 0;
  this.height = 0;
  
  this.x = 0;
  this.y = 0;
  
  this.destinationX = 0;
  this.destinationY = 0;
  
  this.angle = 0;
  
  this.body;
  this.physicsActive;
  
  this.distanceToPlayer = 0;
  
  this.life = 100;
  this.stun = 0;
  
  this.sparks;
  this.explosion;
  
  this.velocityVector;
  
  this.respawned = false;
  
  
  this.profit;
  this.profitText;
  this.profitAnimation;
  this.profitX;
  this.profitY;
  
  
  this.init = function(index) {
    
    this.index = index;
    
    this.width = 30;
    this.height = 60; 
    
    this.x = rand(0, map.SIZE);
    this.y = rand(0, map.SIZE);
    
    this.findNextDestination();
 
    this.angle = Math.atan2(this.destinationY - this.y, this.destinationX - this.x) + (Math.PI / 2);
    
    this.body = physics.createDynamicBox(this, this.x, this.y, this.width, this.height, this.angle, this.LINEAR_DAMPING, this.ANGULAR_DAMPING);
    this.physicsActive = true;
    
    this.sparks = new ParticleSystem();
    this.sparks.setMode(ParticleSystem.BURST_MODE);
    this.sparks.setType(1);
    this.sparks.setA(0.0, 0.0, 0.0, 0.0);
    this.sparks.setLife(25, 35);
    this.sparks.setParticlesPerTick(9);
    
    this.explosion = new ParticleSystem();
    this.explosion.setMode(ParticleSystem.BURST_MODE);
    this.explosion.setType(2);
    this.explosion.setA(0.0, 0.0, 0.0, 0.0);
    this.explosion.setLife(25, 30);
    this.explosion.setParticlesPerTick(20);
    
    this.profit = new Text();
    this.profit.setFont("mediumWhite");
    this.profit.setAlignment(Text.CENTER);
    this.profit.setLineHeight(18);
    this.profitText = "";
    this.profitAnimation = 0;
  };
  
  
  this.findNextDestination = function() {
    var planet = map.planets[rand(0, map.planets.length - 1)];
    var deltaX = planet.x - this.x;
    var deltaY = planet.y - this.y;
    var destinationAngle = Math.atan2(deltaY, deltaX) + (Math.PI / 2);
    var distanceToDestination = Math.sqrt((deltaX * deltaX) + (deltaY * deltaY));
    distanceToDestination -= planet.radius + this.DISTANCE_TO_PLANET;
    this.destinationX = this.x + (Math.sin(destinationAngle) * distanceToDestination);
    this.destinationY = this.y + (-Math.cos(destinationAngle) * distanceToDestination);
  };
  
  
  this.removeLife = function(lifeLoss) {
    if(this.stun < 0.7) {
      this.stun = 1.0;
      this.life -= lifeLoss;
      if(this.life < 0) {

        this.explosion.setEmitter(this.x + 0.0001, this.y + 0.0001);
        this.explosion.setV(
            -150.0 + (this.velocityVector.x * 10), 150.0 + (this.velocityVector.x * 10), 
            -150.0 + (this.velocityVector.y * 10), 150.0 + (this.velocityVector.y * 10)
        );
        this.explosion.burst();
        
        player.giveXP(this.GIVE_XP);
        this.profitText = "xp " + this.GIVE_XP.toString();
        this.profitAnimation = 2.0;
        this.profitX = this.x;
        this.profitY = this.y;
        
        var foundNewSpawnPoint = false;
        while(!foundNewSpawnPoint) {
          this.x = rand(0, map.SIZE);
          this.y = rand(0, map.SIZE);
          this.distanceToPlayer = distance(this.x, this.y, player.x, player.y);
          if(this.distanceToPlayer > player.PHYSICS_DISTANCE) {
            foundNewSpawnPoint = true;
          }
        }
        this.findNextDestination();
        this.angle = Math.atan2(this.destinationY - this.y, this.destinationX - this.x) + (Math.PI / 2);
        this.life = 100;
        this.respawned = true;
      }
    }
  };
  
  
  this.makeSparks = function(x, y) {
    this.sparks.setEmitter(x, y);
    this.sparks.setV(
        -60.0 + (player.velocityVector.x * 8), 60.0 + (player.velocityVector.x * 8), 
        -60.0 + (player.velocityVector.y * 8), 60.0 + (player.velocityVector.y * 8)
    );
    this.sparks.burst();
  };
  
  
  this.update = function() {
    
    if(this.respawned) {
      this.body.SetPosition(new physics.b2Vec2(this.x / 10, this.y / 10));
      this.body.SetAngle(this.angle);
      this.respawned = false;
    }
    
    this.distanceToPlayer = distance(this.x, this.y, player.x, player.y);

    if(this.distanceToPlayer < player.PHYSICS_DISTANCE) {
      
      this.velocityVector = this.body.GetLinearVelocity().Copy();
      
      if(!this.physicsActive) {
        this.body.SetActive(true);
        this.physicsActive = true;
      }
      
      var deltaX = this.destinationX - this.x;
      var deltaY = this.destinationY - this.y;
      var destinationAngle = Math.atan2(deltaY, deltaX) + (Math.PI / 2);
      var distanceToDestination = Math.sqrt((deltaX * deltaX) + (deltaY * deltaY));
      
      if(distanceToDestination < this.DESTINATION_RADIUS) {
        this.findNextDestination();
        
      } else {
        if(this.stun == 0) {
          var forwardForceVector = new physics.b2Vec2(this.FORCE * Math.sin(destinationAngle), this.FORCE * -Math.cos(destinationAngle));
          this.body.ApplyForce(forwardForceVector, new physics.b2Vec2(this.x / 10, this.y / 10));
          var deltaAngle = destinationAngle - this.angle;
          if(deltaAngle > Math.PI) {
            deltaAngle -= 2 * Math.PI;
          } else if(deltaAngle < -Math.PI) {
            deltaAngle += 2 * Math.PI;
          }
          
          if(deltaAngle < -0.001) {
            this.angle -= this.ROTATION_SPEED * timer.delta;
            if(deltaAngle + (this.ROTATION_SPEED * timer.delta) > 0.0) {
              this.angle = destinationAngle;
            }
          } else if(deltaAngle > 0.001) {
            this.angle += this.ROTATION_SPEED * timer.delta;
            if(deltaAngle - (this.ROTATION_SPEED * timer.delta) < 0.0) {
              this.angle = destinationAngle;
            }
          } else {
            this.angle = destinationAngle;
          }
          this.body.SetAngle(this.angle);
        }
      }
      
    } else {
      if(this.physicsActive) {
        this.body.SetActive(false);
        this.physicsActive = false;
      }
      
      var deltaX = this.destinationX - this.x;
      var deltaY = this.destinationY - this.y;
      var destinationAngle = Math.atan2(deltaY, deltaX) + (Math.PI / 2);
      var distanceToDestination = Math.sqrt((deltaX * deltaX) + (deltaY * deltaY));
      
      if(distanceToDestination < this.DESTINATION_RADIUS) {
        this.findNextDestination();
        this.angle = Math.atan2(this.destinationY - this.y, this.destinationX - this.x) + (Math.PI / 2);
        this.body.SetAngle(this.angle);
      } else {
        this.x += Math.sin(destinationAngle) * this.FAKE_SPEED * timer.delta;
        this.y -= Math.cos(destinationAngle) * this.FAKE_SPEED * timer.delta;
        this.body.SetPosition(new physics.b2Vec2(this.x / 10, this.y / 10));
      }
    }
    
    if(this.stun > 0) {
      this.stun = setInRange(this.stun - timer.delta, 0.0, 1.0);
    }
  };
  
  
  this.applyPhysics = function() {
    if(!this.respawned) {
      var position = this.body.GetPosition();
      this.x = position.x * 10;
      this.y = position.y * 10;
      this.angle = this.body.GetAngle();
    }
  };
  
  
  this.draw = function() {
    if(this.distanceToPlayer < player.DRAW_DISTANCE) {
      c.save();
      c.translate(this.x, this.y);
      c.rotate(this.angle);
      c.drawImage(img.get("npc"), 0, 0, 42, 84, -17.5, -35, 35, 70);
      c.restore();
    }
  };
  
  
  this.drawParticles = function() {
    if(this.distanceToPlayer < player.DRAW_DISTANCE) {
      this.sparks.draw();
    }
    this.explosion.draw();
  };
  
  
  this.drawLife = function() {
    if(this.distanceToPlayer < player.DRAW_DISTANCE) {
      if(this.life < 99.9) {
        var deltaX = (this.x - player.x) * player.cameraZoom; 
        var deltaY = (this.y - player.y) * player.cameraZoom;
        var x = (Math.cos(player.cameraAngle) * deltaX) + (-Math.sin(player.cameraAngle) * deltaY);
        var y = (Math.sin(player.cameraAngle) * deltaX) + (Math.cos(player.cameraAngle) * deltaY);
        x += player.POS_IN_SCREEN_X;
        y += player.POS_IN_SCREEN_Y;
        
        c.fillStyle = "#0c0";
        c.fillRect(x - 15, y - (43 * player.cameraZoom), 30 * (this.life / 100), 3);
        c.globalAlpha = 0.5;
        c.fillStyle = "#f00";
        c.fillRect((x - 15) + (30 * (this.life / 100)), y - (43 * player.cameraZoom), 30 * (1 - (this.life / 100)), 3);
        c.globalAlpha = 1.0;
      }
    }
    if(this.profitAnimation > 0.0) {
      var deltaX = (this.profitX - player.x) * player.cameraZoom; 
      var deltaY = (this.profitY - player.y) * player.cameraZoom;
      var x = (Math.cos(player.cameraAngle) * deltaX) + (-Math.sin(player.cameraAngle) * deltaY);
      var y = (Math.sin(player.cameraAngle) * deltaX) + (Math.cos(player.cameraAngle) * deltaY);
      x += player.POS_IN_SCREEN_X;
      y += player.POS_IN_SCREEN_Y;
      
      c.globalAlpha = setInRange(this.profitAnimation, 0.0, 1.0);
      this.profit.setPosition(x, (y - 118) + (30 * this.profitAnimation));
      this.profit.text(this.profitText);
      this.profit.draw();
      this.profitAnimation -= timer.delta;
      c.globalAlpha = 1;
    }
  };
  
  
  this.drawDebug = function() {
    c.save();
    c.translate(this.x / 10, this.y / 10);
    c.rotate(this.angle);
    c.fillStyle = "#00f";
    c.fillRect(-1.5, -3, 3, 6);
    c.restore();
    
    c.save();
    c.translate(this.destinationX / 10, this.destinationY / 10);
    c.fillStyle = "#ff0";
    c.fillRect(-1, -1, 2, 2);
    c.restore();
  };
  
  
  this.getPhysicsType = function() {
    return "npc";
  };
  
}