function Physics() {
  
  this.b2Vec2;
  this.b2AABB;
  this.b2BodyDef;
  this.b2Body;
  this.b2FixtureDef;
  this.b2Fixture;
  this.b2World;
  this.b2MassData;
  this.b2PolygonShape;
  this.b2CircleShape;
  this.b2DebugDraw;
  this.b2MouseJointDef;
  
  this.world;
  
  this.fixDef;
  this.bodyDef;
  
  this.contactListener;
  
  
  this.init = function() {
    this.b2Vec2 = Box2D.Common.Math.b2Vec2;
    this.b2AABB = Box2D.Collision.b2AABB;
    this.b2BodyDef = Box2D.Dynamics.b2BodyDef;
    this.b2Body = Box2D.Dynamics.b2Body;
    this.b2FixtureDef = Box2D.Dynamics.b2FixtureDef;
    this.b2Fixture = Box2D.Dynamics.b2Fixture;
    this.b2World = Box2D.Dynamics.b2World;
    this.b2MassData = Box2D.Collision.Shapes.b2MassData;
    this.b2PolygonShape = Box2D.Collision.Shapes.b2PolygonShape;
    this.b2CircleShape = Box2D.Collision.Shapes.b2CircleShape;
    this.b2DebugDraw = Box2D.Dynamics.b2DebugDraw;
    this.b2MouseJointDef = Box2D.Dynamics.Joints.b2MouseJointDef;
    
    Box2D.Common.b2Settings.b2_maxTranslation = 4.0;
    
    this.fixDef = new this.b2FixtureDef();
    this.bodyDef = new this.b2BodyDef();
  };
  
  
  this.setup = function() {
    this.world = new this.b2World(new this.b2Vec2(0, 0), true);
    
    this.fixDef.density = 0.0005;
    this.fixDef.friction = 0.1;
    this.fixDef.restitution = 0.9;
    
    this.contactListener = new Box2D.Dynamics.b2ContactListener();
    this.contactListener.PostSolve = function(contact, impulse) {
      physics.collision(contact, impulse);
    }
    this.world.SetContactListener(this.contactListener);
  };
  
  
  this.collision = function(contact, impulse) {
    var objectA = contact.GetFixtureA().GetBody().GetUserData();
    var typeA = objectA.getPhysicsType();
    var objectB = contact.GetFixtureB().GetBody().GetUserData();
    var typeB = objectB.getPhysicsType();
    
    if(typeA == "player" || typeB == "player") {
      var target = objectB;
      if(typeB == "player") {
        var target = objectA;
      }
      
      if(target.getPhysicsType() == "npc") {
        var totalImpulse = impulse.normalImpulses[0] + impulse.normalImpulses[1];
        if(totalImpulse > 0.15) {
          target.removeLife(totalImpulse * 80);
        }
        if(totalImpulse > 0.04) {
          var worldManifold = new Box2D.Collision.b2WorldManifold();
          contact.GetWorldManifold(worldManifold);
          var contactPoint = worldManifold.m_points[0];
          target.makeSparks(contactPoint.x * 10, contactPoint.y * 10);
        }
      }
    }
    if(typeA == "shot" || typeB == "shot") {
      var target = objectB;
      var shot = objectA;
      if(typeB == "shot") {
        var target = objectA;
        var shot = objectB;
      }
      shot.life = 0;
      if(target.getPhysicsType() == "npc") {
        var worldManifold = new Box2D.Collision.b2WorldManifold();
        contact.GetWorldManifold(worldManifold);
        var contactPoint = worldManifold.m_points[0];
        target.makeSparks(contactPoint.x * 10, contactPoint.y * 10);
        target.removeLife(shot.damage);

      }
    }
  };
  
  
  this.createStaticBox = function(object, x, y, w, h, angle) {
    this.bodyDef.type = this.b2Body.b2_staticBody;
    this.fixDef.shape = new this.b2PolygonShape();
    this.fixDef.shape.SetAsBox(w / 20, h / 20);
    this.bodyDef.position.Set(x / 10, y / 10);
    var body = this.world.CreateBody(this.bodyDef);
    body.CreateFixture(this.fixDef);
    body.SetAngle(angle);
    body.SetUserData(object);
    return body;
  };
  
  
  this.createDynamicBox = function(object, x, y, w, h, angle, linearDamping, angularDamping) {
    this.bodyDef.type = this.b2Body.b2_dynamicBody;
    this.bodyDef.linearDamping = linearDamping;
    this.bodyDef.angularDamping = angularDamping;
    this.fixDef.shape = new this.b2PolygonShape();
    this.fixDef.shape.SetAsBox(w / 20, h / 20);
    this.bodyDef.position.Set(x / 10, y / 10);
    var body = this.world.CreateBody(this.bodyDef);
    body.CreateFixture(this.fixDef);
    body.SetAngle(angle);
    body.SetUserData(object);
    return body;
  }; 
  
  
  this.createStaticCircle = function(object, x, y, r) {
    this.bodyDef.type = this.b2Body.b2_staticBody;
    this.fixDef.shape = new this.b2CircleShape(r / 10);
    this.bodyDef.position.Set(x / 10, y / 10);
    var body = this.world.CreateBody(this.bodyDef);
    body.CreateFixture(this.fixDef);
    body.SetUserData(object);
    return body;
  };
  
  
  this.createDynamicCircle = function(object, x, y, r, linearDamping, angularDamping) {
    this.bodyDef.type = this.b2Body.b2_dynamicBody;
    this.bodyDef.linearDamping = linearDamping;
    this.bodyDef.angularDamping = angularDamping;
    this.fixDef.shape = new this.b2CircleShape(r / 10);
    this.bodyDef.position.Set(x / 10, y / 10);
    var body = this.world.CreateBody(this.bodyDef);
    body.CreateFixture(this.fixDef);
    body.SetUserData(object);
    return body;
  };
  
  
  this.update = function() {
    this.world.Step(0.020, 10, 10);
    this.world.ClearForces();
  };
  
  
};

