function Planet() {
  
  this.ORBIT_DISTANCE_MIN = 40;
  this.ORBIT_DISTANCE_MAX = 80;
  
  this.ORBIT_SPEED_MIN = 0.3;
  this.ORBIT_SPEED_MAX = 1.2;
  
  this.INFO_DISTANCE = 80;
  
  this.ALPHA = ["a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z"];
  
  this.type;
  this.questLevel;
  this.hasMoon;
  this.moonHasQuest;
  
  this.index;
  
  this.x;
  this.y;
  
  this.radius;
  this.rotation;
  
  this.isMoon = false;
  
  this.animationOffset;
  
  this.parentPlanet = -1;
  this.orbitDistance = 0;
  this.orbitSpeed = 0;
  this.orbit = 0;
  
  this.body;
  this.physicsActive;
  
  this.distanceToPlayer = 0;
  
  this.name;
  
  this.label;
  this.labelAnimation;
  
  this.profit;
  this.profitText;
  this.profitAnimation;


  this.init = function(type, questLevel, radius, hasMoon, moonHasQuest) {
    this.type = type;
    this.questLevel = questLevel;
    this.radius = radius;
    this.hasMoon = hasMoon;
    this.moonHasQuest = moonHasQuest;
    
    this.name = this.ALPHA[rand(0, 25)] + this.ALPHA[rand(0, 25)] + rand(10, 999);
    
    this.rotation = randFloat(0, 2 * Math.PI);
    
    this.label = new Text();
    this.label.setFont("smallWhite");
    this.labelAnimation = 0;
    
    this.profit = new Text();
    this.profit.setFont("mediumWhite");
    this.profit.setAlignment(Text.CENTER);
    this.profit.setLineHeight(18);
    this.profitText = "";
    this.profitAnimation = 0;
  };
  
  
  this.place = function(index, x, y, parentPlanet) {
    
    this.index = index;
    
    this.x = x;
    this.y = y;
    
    this.parentPlanet = parentPlanet;
    
    this.animationOffset = randFloat(0, 2 * Math.PI);
    
    if(this.parentPlanet != -1) {
      this.isMoon = true;
      this.orbitDistance = map.planets[this.parentPlanet].radius + this.radius;
      this.orbitDistance += rand(this.ORBIT_DISTANCE_MIN, this.ORBIT_DISTANCE_MAX);
      this.orbitSpeed = randFloat(this.ORBIT_SPEED_MIN, this.ORBIT_SPEED_MAX);
      this.orbit = randFloat(0, 2 * Math.PI);
      this.updateMoonPos();
    }
    
    this.body = physics.createStaticCircle(this, this.x, this.y, this.radius);
    this.physicsActive = true;
  };
  
  
  this.reportProfit = function(xp, money, bonus) {
    this.profitText = "$" + money.toString() + "\nxp " + xp.toString();
    if(bonus) {
      this.profitText = "bonus!\n" + this.profitText;
    }
    this.profitAnimation = 2.0;
  };
  
  
  this.update = function() {

    this.distanceToPlayer = distance(this.x, this.y, player.x, player.y);

    if(this.distanceToPlayer < player.PHYSICS_DISTANCE) {
      
      if(!this.physicsActive) {
        this.body.SetActive(true);
        this.physicsActive = true;
      }
      
      if(this.isMoon) {
        this.orbit += this.orbitSpeed * timer.delta;
        if(this.orbit > 2 * Math.PI) {
          this.orbit -= 2 * Math.PI;
        }
        this.updateMoonPos();
        this.body.SetPosition(new physics.b2Vec2(this.x / 10, this.y / 10));
      }
      
      if(this.distanceToPlayer - this.radius < player.APPROACH_DISTANCE) {
        quest.reportApproach(this.index);
      }
      
      if(this.distanceToPlayer < ((this.INFO_DISTANCE / player.baseZoom) + this.radius)) {
        if(this.labelAnimation < 0.6) {
          this.labelAnimation += timer.delta;
        }
        this.labelAnimation = setInRange(this.labelAnimation, 0.0, 0.6);
      }
      
    } else if(this.physicsActive) {
      this.body.SetActive(false);
      this.physicsActive = false;
    }
  };
  
  
  this.updateMoonPos = function() {
    var parentPlanetObj = map.planets[this.parentPlanet];
    this.x = parentPlanetObj.x + (Math.cos(this.orbit) * this.orbitDistance);
    this.y = parentPlanetObj.y + (Math.sin(this.orbit) * this.orbitDistance);
  };
  
  
  this.draw = function() {
    if(this.distanceToPlayer < player.DRAW_DISTANCE) {
      if(this.type < 50) {
        c.save();
        c.translate(this.x, this.y);
        c.rotate(this.rotation);
        c.drawImage(img.get("planets"), 0 + (150 * (this.type - 1)), 0, 150, 150, -62.5, -62.5, 125, 125);
        c.restore();
        
      } else {
        c.save();
        c.translate(this.x, this.y);
        c.rotate(this.rotation + this.orbit);
        c.drawImage(img.get("moons"), 0 + (50 * (this.type - 51)), 0, 50, 50, -21.8, -21.8, 42.6, 42.6);
        c.restore();
      }
    }
  };
  
  
  this.drawInfo = function() {
    if(this.distanceToPlayer < player.DRAW_DISTANCE) {
      var deltaX = (this.x - player.x) * player.cameraZoom; 
      var deltaY = (this.y - player.y) * player.cameraZoom;
      var x = (Math.cos(player.cameraAngle) * deltaX) + (-Math.sin(player.cameraAngle) * deltaY);
      var y = (Math.sin(player.cameraAngle) * deltaX) + (Math.cos(player.cameraAngle) * deltaY);
      x += player.POS_IN_SCREEN_X;
      y += player.POS_IN_SCREEN_Y;
      
      if(this.distanceToPlayer < ((this.INFO_DISTANCE / player.baseZoom) + this.radius)) {
        
        var arcProgress = 1/4 * Math.PI;
        var connection1 = 1.0;
        var connection2 = 1.0;
        var showLetters = 0;
        
        if(this.labelAnimation < 0.2) {
          arcProgress *= this.labelAnimation / 0.2;
          connection1 = 0.0;
          connection2 = 0.0;
          
        } else if(this.labelAnimation < 0.3) {
          connection1 = (this.labelAnimation - 0.2) / 0.1;
          connection2 = 0.0;
          
        } else if(this.labelAnimation < 0.4) {
          connection1 = 1.0;
          connection2 = (this.labelAnimation - 0.3) / 0.1;
          
        } else {
          showLetters = Math.ceil(5 * ((this.labelAnimation - 0.4) / 0.2));
        }
        
        var labelX = (this.radius * player.cameraZoom) + 5;
        var labelY = (this.radius * player.cameraZoom) + 5;
        var cos45 = Math.cos(Math.PI / 4);
        connection1 *= 1.0 - cos45;

        c.strokeStyle = "#000";
        c.globalAlpha = 0.8;
        c.lineWidth = 3.5;
        c.beginPath();
        c.arc(x, y, (this.radius * player.cameraZoom) + 5, -(Math.PI / 4 + arcProgress), -(Math.PI / 4 - arcProgress), false);
        c.stroke();
        c.beginPath();
        c.moveTo(x + (labelX * cos45), y - (labelY * cos45));
        c.lineTo(x + (labelX * (cos45 + connection1)), y - (labelY * (cos45 + connection1)));
        c.stroke();
        if(connection2 > 0.0) {
          c.beginPath();
          c.moveTo(x + labelX, y - labelY);
          c.lineTo(x + labelX + (20 * connection2), y - labelY);
          c.stroke();
        }
        
        c.globalAlpha = 1.0;
        c.strokeStyle = "#fff";
        c.lineWidth = 1.5;
        c.beginPath();
        c.arc(x, y, (this.radius * player.cameraZoom) + 5, -(Math.PI / 4 + arcProgress), -(Math.PI / 4 - arcProgress), false);
        c.stroke();
        c.beginPath();
        c.moveTo(x + (labelX * cos45), y - (labelY * cos45));
        c.lineTo(x + (labelX * (cos45 + connection1)), y - (labelY * (cos45 + connection1)));
        c.stroke();
        if(connection2 > 0.0) {
          c.beginPath();
          c.moveTo(x + labelX, y - labelY);
          c.lineTo(x + labelX + (20 * connection2), y - labelY);
          c.stroke();
        }
        
        c.lineWidth = 1;
        
        if(showLetters > 0) {
          this.label.setPosition(x + labelX, y - (labelY + 14));
          this.label.text(this.name.substring(0, showLetters));
          this.label.draw();
        }

      } else {
        this.labelAnimation = 0;
      }
      
      if(this.profitAnimation > 0.0) {
        c.globalAlpha = setInRange(this.profitAnimation, 0.0, 1.0);
        this.profit.setPosition(x, (y - (this.radius + 106)) + (30 * this.profitAnimation));
        this.profit.text(this.profitText);
        this.profit.draw();
        this.profitAnimation -= timer.delta;
        c.globalAlpha = 1;
      }
    }
  };
  
  
  this.drawDebug = function() {
    c.beginPath();
    c.arc(this.x / 10, this.y / 10, this.radius / 10, 0, 2 * Math.PI, false);
    if(this.index == quest.currentQuest) {
      c.fillStyle = "#fff";
    } else {
      c.fillStyle = "#f00";
    }
    c.fill();
    c.closePath();
  };
  
  
  this.getPhysicsType = function() {
    return "planet";
  };
  
}


Planet.types = [
  { type :  1, questLevel : 1, radius : 20},
  { type :  2, questLevel : 0, radius : 25},
  { type :  3, questLevel : 0, radius : 30},
  { type :  4, questLevel : 0, radius : 30},
  { type :  5, questLevel : 1, radius : 35},
  { type :  6, questLevel : 0, radius : 35},
  { type :  7, questLevel : 0, radius : 40},
  { type :  8, questLevel : 1, radius : 50},
  { type :  9, questLevel : 0, radius : 25},
  { type : 10, questLevel : 1, radius : 30},
  { type : 11, questLevel : 0, radius : 35},
  { type : 12, questLevel : 1, radius : 45},
];

Planet.moonTypes = [
  { type : 51, questLevel : 2, radius : 16},
  { type : 52, questLevel : 2, radius : 12},
  { type : 53, questLevel : 2, radius : 8},
  { type : 54, questLevel : 2, radius : 10},
  { type : 55, questLevel : 3, radius : 6},
  { type : 56, questLevel : 3, radius : 14},
];