function Player() {
  
  this.WIDTH = 30;
  this.HEIGHT = 60;  
  
  this.POS_IN_SCREEN_X = 400;
  this.POS_IN_SCREEN_Y = 450;
  
  this.BACKWARD_FORCE_RATIO = 0.6;
  this.LINEAR_DAMPING = 4.0;
  this.ANGULAR_DAMPING = 12.0;
  
  this.PHYSICS_DISTANCE = 1200;
  this.DRAW_DISTANCE = 1200;
  this.APPROACH_DISTANCE = 50;
  this.MIN_SPAWN_DISTANCE = 300;
  this.JUMP_TO_OTHER_SIDE_OF_WORLD = 920;
  
  this.TRACE_LENGTH = 100;
  
  this.NEXT_LEVEL_XP = [0, 20, 50, 80, 150, 240, 400, 640]; //600, 1000, 2000];
  
  this.CAMERA_ROTATION_LAG = 15;
  
  this.START_MONEY = 0;
  
  this.x = 0;
  this.y = 0;
  
  this.unlockedGun = false;
  
  
  this.angle = 0;
  
  this.body;
  
  this.forwardForce = 1.0; // lowest: 1.0, highest : 3.0
  this.rotationSpeed = 1.2; // lowest: 1.2, highest : 3.0
  this.timeToTopRotationSpeed = 0.2; // lowest: 0.2, highest : 0.05
  this.baseZoom = 1.2; // lowest: 1.2, highest: 0.8;
  this.fireRate = 1.4; // lowest: 1.0, highest: 3.0;
  this.shotDamage = 9.0 // lowest: 10.0, highest: 50;
  
  this.velocity = 0.0;
  this.angularVelocity = 0.0;
  this.lastVelocities = [];
  this.averageVelocity = 0.0;
  this.moveDirection = 0;
  this.jetImage;
  
  this.trace = [];
  this.traceCounter = 0;
  
  this.level;
  this.xp;
  this.money;
  this.tempMoney;
  
  this.lastAngles = [];
  
  this.cameraZoom;
  this.cameraAngle;
  
  this.tempZoom = this.baseZoom;
  
  this.velocityVector;
  
  this.shots = {};
  this.shotsCounter = 0;
  this.shotsCooldown = 0;
  
  
  this.init = function() {
    
    var foundSpawn = false;
    var planet;
    while(!foundSpawn) {
      this.x = rand(0.3 * map.SIZE, 0.7 * map.SIZE);
      this.y = rand(0.3 * map.SIZE, 0.7 * map.SIZE);
      foundSpawn = true;
      for(var i = 0; i < map.numOfPlanets; i++) {
        planet = map.planets[i];
        if(distance(this.x, this.y, planet.x, planet.y) < this.MIN_SPAWN_DISTANCE) {
          foundSpawn = false;
        }
      }
    }
    
    this.angle = randFloat(0, 2 * Math.PI);
    
    this.body = physics.createDynamicBox(this, this.x, this.y, this.WIDTH, this.HEIGHT, this.angle, this.LINEAR_DAMPING, this.ANGULAR_DAMPING);
    this.body.SetSleepingAllowed(false);
    
    for(var i = 0; i < this.TRACE_LENGTH; i++) {
      this.trace[i] = this.getTraceSegment(0);
    }
    
    this.level = 1;
    this.xp = 0;
    this.money = this.START_MONEY;
    this.tempMoney = 0;
    this.jetImage = rand(0, 7);
  };
  
  
  this.giveXP = function(xp) {
    this.xp += xp;
    if(this.xp >= this.NEXT_LEVEL_XP[this.level]) {
      if(this.level == this.NEXT_LEVEL_XP.length - 1) {
        this.xp = this.NEXT_LEVEL_XP[this.level];
        this.level++;
        
        panel.shopUnlockTutorial = 5.0;
        panel.shopUnlockText = "you've reached level 8. well done!";
        
      } else {
        this.xp -= this.NEXT_LEVEL_XP[this.level];
        this.level++;
        
        if(this.level == 2) {
          this.unlockedGun = true;
          panel.shootTutorial = 4.0;
          upgradeWindow.upgradesUnlocked[4] = true;
          upgradeWindow.upgradesUnlocked[5] = true;
          
        } else if(this.level == 3) {
          panel.shopUnlockTutorial = 5.0;
          panel.shopUnlockText = "you've reached level 3 and unlocked green deliveries.";
          quest.allowedQuests[1] = true;
          
        } else if(this.level == 4) {
          panel.shopUnlockTutorial = 5.0;
          panel.shopUnlockText = "you've reached level 4 and unlocked yellow deliveries.";
          quest.allowedQuests[2] = true;
          
        } else if(this.level == 5) {
          panel.shopUnlockTutorial = 5.0;
          panel.shopUnlockText = "you've reached level 5 and unlocked red deliveries.";
          quest.allowedQuests[3] = true;
          
        }
      }
    }
  };
  
  
  this.giveMoney = function(money) {
    this.tempMoney += money;
  };
  
  
  this.applyCamera = function() {
    this.cameraZoom = this.baseZoom * (1.00 - (0.1 * (this.averageVelocity / 80.0)));
    this.cameraAngle = 0;
    for(var i = 0; i < this.lastAngles.length; i++) {
      this.cameraAngle -= this.lastAngles[i];
    }
    this.cameraAngle /= this.lastAngles.length;
    c.save();
    c.translate(this.POS_IN_SCREEN_X, this.POS_IN_SCREEN_Y);
    c.scale(this.cameraZoom, this.cameraZoom);
    c.rotate(this.cameraAngle);
    c.translate(-this.x, -this.y);
  };
  
  
  this.removeCamera = function() {
    c.restore();
  };
  
  
  this.update = function() {
    
    if(this.x < -this.JUMP_TO_OTHER_SIDE_OF_WORLD) {
      this.x += map.SIZE + (2 * this.JUMP_TO_OTHER_SIDE_OF_WORLD);
      this.body.SetPosition(new physics.b2Vec2(this.x / 10, this.y / 10));
    } else if(this.x > (map.SIZE + this.JUMP_TO_OTHER_SIDE_OF_WORLD)) {
      this.x -= map.SIZE + (2 * this.JUMP_TO_OTHER_SIDE_OF_WORLD);
      this.body.SetPosition(new physics.b2Vec2(this.x / 10, this.y / 10));
    }
    if(this.y < -this.JUMP_TO_OTHER_SIDE_OF_WORLD) {
      this.y += map.SIZE + (2 * this.JUMP_TO_OTHER_SIDE_OF_WORLD);
      this.body.SetPosition(new physics.b2Vec2(this.x / 10, this.y / 10));
    } else if(this.y > (map.SIZE + this.JUMP_TO_OTHER_SIDE_OF_WORLD)) {
      this.y -= map.SIZE + (2 * this.JUMP_TO_OTHER_SIDE_OF_WORLD);
      this.body.SetPosition(new physics.b2Vec2(this.x / 10, this.y / 10));
    }
    
    var forwardVector = new physics.b2Vec2(Math.sin(this.angle), -Math.cos(this.angle));
    this.velocityVector = this.body.GetLinearVelocity().Copy();
    this.velocity = this.velocityVector.Copy().Normalize();
    this.lastVelocities.push(this.velocity);
    if(this.lastVelocities.length > 35) {
      this.lastVelocities.shift();
    }
    this.averageVelocity = 0;
    for(var i = 0; i < this.lastVelocities.length; i++) {
      this.averageVelocity += this.lastVelocities[i];
    }
    this.averageVelocity /= this.lastVelocities.length;
    
    var steering = this.angle;
    
    var angularAcceleration = this.rotationSpeed * (timer.delta / this.timeToTopRotationSpeed);
    
    this.moveDirection = 0;
    
    if((keyboard.isPressed(Keyboard.ARROW_UP) || keyboard.isPressed(Keyboard.W)) &&
       (!keyboard.isPressed(Keyboard.ARROW_DOWN) && !keyboard.isPressed(Keyboard.S))) {
      this.moveDirection = 1;
    }
    if((keyboard.isPressed(Keyboard.ARROW_DOWN) || keyboard.isPressed(Keyboard.S)) &&
       (!keyboard.isPressed(Keyboard.ARROW_UP) && !keyboard.isPressed(Keyboard.W))) {
      this.moveDirection = -1;
    }
    
    if((keyboard.isPressed(Keyboard.ARROW_LEFT) || keyboard.isPressed(Keyboard.A)) &&
       (!keyboard.isPressed(Keyboard.ARROW_RIGHT) && !keyboard.isPressed(Keyboard.D))) {
      if(this.moveDirection == -1) {
        this.angularVelocity += angularAcceleration;
      } else {
        this.angularVelocity -= angularAcceleration;
      }
      
    } else if((keyboard.isPressed(Keyboard.ARROW_RIGHT) || keyboard.isPressed(Keyboard.D)) &&
       (!keyboard.isPressed(Keyboard.ARROW_LEFT) && !keyboard.isPressed(Keyboard.A))) {
      if(this.moveDirection == -1) {
        this.angularVelocity -= angularAcceleration;
      } else {
        this.angularVelocity += angularAcceleration;
      }
      
    } else {
      if(this.angularVelocity < 0) {
        this.angularVelocity += angularAcceleration;
        if(this.angularVelocity >= 0) {
          this.angularVelocity = 0;
        }
      } else if(this.angularVelocity > 0) {
        this.angularVelocity -= angularAcceleration;
        if(this.angularVelocity <= 0) {
          this.angularVelocity = 0;
        }
      }
    }
    if(this.angularVelocity <= -this.rotationSpeed) {
      this.angularVelocity = -this.rotationSpeed;
    }
    if(this.angularVelocity >= this.rotationSpeed) {
      this.angularVelocity = this.rotationSpeed;
    }
    
    steering += this.angularVelocity * timer.delta;
    this.body.SetAngle(steering);
    
    var forwardForceVector = new physics.b2Vec2(
      this.forwardForce * Math.sin(steering),
      -this.forwardForce * Math.cos(steering)
    );
    
    var backwardForceVector = new physics.b2Vec2(
      -this.BACKWARD_FORCE_RATIO * this.forwardForce * Math.sin(steering),
      this.BACKWARD_FORCE_RATIO * this.forwardForce * Math.cos(steering)
    );
    
    if(this.moveDirection == 1) {
      this.body.ApplyForce(forwardForceVector, new physics.b2Vec2(this.x / 10, this.y / 10));
    }
    if(this.moveDirection == -1) {
      this.body.ApplyForce(backwardForceVector, new physics.b2Vec2(this.x / 10, this.y / 10));
    }
    
    var minusOpacity = 1 / this.TRACE_LENGTH;
    for(var i = 0; i < this.TRACE_LENGTH; i++) {
      this.trace[i].opacity -= minusOpacity;
    }
    
    this.jetImage = rand(0, 7);
    
    this.lastAngles.push(this.angle);
    if(this.lastAngles.length > this.CAMERA_ROTATION_LAG) {
      this.lastAngles.shift();
    }
    
    if(this.tempZoom < this.baseZoom) {
      this.baseZoom -= 0.1 * timer.delta;
    }
    
    if(this.shotsCooldown > 0) {
      this.shotsCooldown = setInRange(this.shotsCooldown - timer.delta, 0.0, 100.0);
    } else {
      if(this.unlockedGun && keyboard.isPressed(Keyboard.SPACE_BAR)) {
        if(panel.shootTutorial == 4.0) {
          panel.shootTutorial = 3.0;
        }
        this.shotsCooldown = 1.0 / this.fireRate;
        this.shots[this.shotsCounter] = new Shot();
        this.shots[this.shotsCounter].init(this.shotsCounter, this.x, this.y, this.angle, this.velocityVector, this.shotDamage);
        this.shotsCounter++;
      }
    }
  };
  
  
  this.updateMoney = function() {
    if(this.tempMoney > 0) {
      if(this.tempMoney >= 7) {
        this.money += 7;
        this.tempMoney -= 7;
      } else {
        this.money += this.tempMoney;
        this.tempMoney = 0;
      }
      if(panel.window == upgradeWindow) {
        upgradeWindow.updateButtons();
      }
    }
  };
  
  
  this.applyPhysics = function() {
    var position = this.body.GetPosition();
    this.x = position.x * 10;
    this.y = position.y * 10;
    this.angle = this.body.GetAngle();
    
    this.traceCounter = (this.traceCounter + 1) % this.TRACE_LENGTH;
    this.trace[this.traceCounter] = this.getTraceSegment(this.velocity / 100);
    
    for(var shotId in this.shots) {
      this.shots[shotId].applyPhysics();
    }
  };
  
  
  this.draw = function() {
    c.save();
    c.translate(this.x, this.y);
    c.rotate(this.angle);
    c.drawImage(img.get("player"), 0, 0, 84, 84, -35, -35, 70, 70);
    if(this.moveDirection == 1) {
      c.drawImage(img.get("jet"), 72 * this.jetImage, 0, 72, 144, -30, -60, 60, 120);
    } else if(this.moveDirection == -1) {
      c.drawImage(img.get("jet"), 72 * this.jetImage, 144, 72, 144, -30, -60, 60, 120);
    }
    c.restore();
    
    for(var shotId in this.shots) {
      if(this.shots[shotId].life < 0) {
        this.shots[shotId].deleteMe();
        delete this.shots[shotId];
      } else {
        this.shots[shotId].draw();
      }
    }
  };
  
  
  this.drawDebug = function() {
    c.save();
    c.translate(this.x / 10, this.y / 10);
    c.rotate(this.angle);
    c.fillStyle = "#0f0";
    c.fillRect(-1.5, -3, 3, 6);
    c.restore();
    
    c.fillStyle = "#ff0";
    c.fillRect(0, 595, this.velocity * 10, 5);
  };
  
  
  this.drawTrace = function() {
    var index1;
    var index2;
    var segment1;
    var segment2;
    for(var i = 0; i < this.TRACE_LENGTH; i++) {
      index1 = i;
      index2 = index1 - 1;
      if(index2 == -1) {
        index2 = this.TRACE_LENGTH - 1;
      }
      if(index2 != this.traceCounter) {
        segment1 = this.trace[index1];
        segment2 = this.trace[index2];
        if(segment1.opacity > 0 && distance(segment1.x1, segment1.y1, segment2.x1, segment2.y1) < 1000) {
          c.globalAlpha = segment1.opacity;
          c.beginPath();
          c.moveTo(segment1.x1, segment1.y1);
          c.lineTo(segment2.x1, segment2.y1);
          c.moveTo(segment1.x2, segment1.y2);
          c.lineTo(segment2.x2, segment2.y2);
          c.closePath();
          c.strokeStyle = "#fff";
          c.stroke();
        }
      }
    }
    c.globalAlpha = 1;
  };
  
  
  this.getTraceSegment = function(opacity) {
    var x1 = this.x - (Math.sin(this.angle) * 30) - (Math.cos(this.angle) * 25);
    var y1 = this.y + (Math.cos(this.angle) * 30) - (Math.sin(this.angle) * 25);
    var x2 = this.x - (Math.sin(this.angle) * 30) + (Math.cos(this.angle) * 25);
    var y2 = this.y + (Math.cos(this.angle) * 30) + (Math.sin(this.angle) * 25);
    return { x1 : x1, y1 : y1, x2 : x2, y2 : y2, opacity : opacity };
  };
  
  
  this.getPhysicsType = function() {
    return "player";
  };
  
}