function Quest() {
  
  this.ANIMATION_STEP = Math.PI * 0.8;
  this.LEVEL_COLORS = ["#99f", "#9f9", "#ff9", "#f99"];
  this.MIN_QUEST_DISTANCE = [1400, 1700, 2200, 4000];
  this.MAX_QUEST_DISTANCE = [2400, 2700, 3200, 5000];
  
  this.MONEY_FOR_QUEST = [400, 600, 800, 1000]; //[200, 300, 500, 1000];
  this.XP_FOR_QUEST = [10, 15, 24, 36];
  this.TIME_FOR_BONUS = [6, 6, 6, 6];
  this.MONEY_FOR_BONUS = [200, 300, 400, 800]; //[100, 200, 400, 800];
  this.XP_FOR_BONUS = [5, 10, 20, 40];
  
  
  this.questTargets;
  
  this.hasQuest;
  this.allowedQuests;
  
  this.currentQuest;
  this.currentQuestLevel;
  this.bonusCountDown;
  
  this.animation;
  
  this.approachPlanet;
  this.approachStart;
  this.approachAnimation;
  
  
  this.init = function() {
    
    this.questTargets = [];
    
    for(var i = 0; i < map.numOfPlanets; i++) {
      if(map.planets[i].questLevel == -1 && !map.planets[i].moonHasQuest) {
        this.questTargets.push(i);
      }
    }
    
    this.hasQuest = false;
    this.allowedQuests = [true, false, false, false];
    
    this.currentQuest = -1;
    this.bonusCountDown = 0;
    
    this.animation = 0;
    
    this.approachPlanet = -1;
    this.approachStart = 0;
    this.approachAnimation = 0;
    
    this.approach2Planet = -1;
    this.approach2Start = 0;
    this.approach2Animation = 0;
  };
  
  
  this.update = function() {
    this.animation += this.ANIMATION_STEP * timer.delta;
    if(this.bonusCountDown > 0) {
      this.bonusCountDown -= timer.delta;
    }
    if(this.approachPlanet != -1) {
      this.approachAnimation += timer.delta;
    }
    if(this.approach2Planet != -1) {
      this.approach2Animation += timer.delta;
    }
  };
  
  
  this.reportApproach = function(index) {
    
    var planet = map.planets[index];
    
    if(!this.hasQuest) {
      if(planet.questLevel != -1 && this.allowedQuests[planet.questLevel]) {
        this.hasQuest = true;
        this.currentQuestLevel = planet.questLevel;
        var foundTarget = false;
        var tries = 0;
        while(!foundTarget || tries > 200) {
          this.currentQuest = this.questTargets[rand(0, this.questTargets.length - 1)];
          var targetPlanet = map.planets[this.currentQuest];
          var targetDistance = distance(player.x, player.y, targetPlanet.x, targetPlanet.y);
          if(targetDistance > this.MIN_QUEST_DISTANCE[this.currentQuestLevel] && 
             targetDistance < this.MAX_QUEST_DISTANCE[this.currentQuestLevel]) {
            foundTarget = true;
          }
          tries++;
        }
        this.bonusCountDown = this.TIME_FOR_BONUS[this.currentQuestLevel];
        this.approachPlanet = index;
        this.approachStart = planet.radius + (8 * Math.abs(Math.sin(this.animation + planet.animationOffset)));
        this.approachAnimation = 0;
      }
      
    } else {
      if(index == this.currentQuest) {
        
        var xp = this.XP_FOR_QUEST[this.currentQuestLevel];
        var money = this.MONEY_FOR_QUEST[this.currentQuestLevel];
        var bonus = this.bonusCountDown > 0;
        if(bonus) {
          xp += this.XP_FOR_BONUS[this.currentQuestLevel];
          money += this.MONEY_FOR_BONUS[this.currentQuestLevel];
        }
        player.giveXP(xp);
        player.giveMoney(money);
        planet.reportProfit(xp, money, bonus);
        
        this.hasQuest = false;
        this.currentQuest = -1;
        
        this.approach2Planet = index;
        this.approach2Start = planet.radius + (8 * Math.abs(Math.sin(this.animation + planet.animationOffset)));
        this.approach2Animation = 0;
      }
    }
  };
  
  
  this.draw = function() {
    var planet;
    var radius;
    if(!this.hasQuest) {
      c.globalAlpha = 0.3;
      for(var i = 0; i < map.numOfPlanets; i++) {
        planet = map.planets[i];
        if(planet.distanceToPlayer < player.DRAW_DISTANCE && 
            planet.questLevel != -1 && 
            this.allowedQuests[planet.questLevel]) {
          radius = planet.radius + (8 * Math.abs(Math.sin(this.animation + planet.animationOffset)));
          c.beginPath();
          c.arc(planet.x, planet.y, radius + 16, 0, 2 * Math.PI, false);
          c.arc(planet.x, planet.y, radius + 8, 0, 2 * Math.PI, true);
          c.closePath();
          c.fillStyle = this.LEVEL_COLORS[planet.questLevel];
          c.fill();
        }
      }
      c.globalAlpha = 1.0;
      
    } else {
      planet = map.planets[this.currentQuest];
      if(planet.distanceToPlayer < player.DRAW_DISTANCE) {
        radius = planet.radius + (8 * Math.abs(Math.sin(this.animation + planet.animationOffset)));
        var numOfSegments = 4 + (2 * Math.ceil(planet.radius / 10));
        c.globalAlpha = 0.5;
        for(var a = 0; a < numOfSegments; a++) {
          var angle = a * 2 * (Math.PI / numOfSegments);
          c.beginPath();
          c.moveTo(planet.x + ( 7 * Math.sin(angle)) + ((radius + 24) * Math.cos(angle)), planet.y + ( 7 * Math.cos(angle)) - ((radius + 24) * Math.sin(angle)));
          c.lineTo(planet.x + (-7 * Math.sin(angle)) + ((radius + 24) * Math.cos(angle)), planet.y + (-7 * Math.cos(angle)) - ((radius + 24) * Math.sin(angle)));
          c.lineTo(planet.x                           + ((radius +  8) * Math.cos(angle)), planet.y                           - ((radius +  8) * Math.sin(angle)));
          c.closePath();
          c.fillStyle = this.LEVEL_COLORS[this.currentQuestLevel];
          c.fill();
        }
        c.globalAlpha = 1.0;
      }
    }
    
    if(this.approachPlanet != -1) {
      planet = map.planets[this.approachPlanet];
      radius = this.approachStart + (100 * this.approachAnimation);
      c.globalAlpha = setInRange(0.6 * (1.0 - (this.approachAnimation / 0.3)), 0.0, 1.0);
      c.beginPath();
      c.arc(planet.x, planet.y, radius + 16, 0, 2 * Math.PI, false);
      c.arc(planet.x, planet.y, radius + 8, 0, 2 * Math.PI, true);
      c.closePath();
      c.fillStyle = this.LEVEL_COLORS[planet.questLevel];
      c.fill();
      c.globalAlpha = 1.0;
      if(this.approachAnimation >= 0.3) {
        this.approachPlanet = -1;
      }
    }
    
    if(this.approach2Planet != -1) {
      planet = map.planets[this.approach2Planet];
      radius = this.approach2Start + (100 * this.approach2Animation);
      c.globalAlpha = setInRange(0.8 * (1.0 - (this.approach2Animation / 0.3)), 0.0, 1.0);
      var numOfSegments = 4 + (2 * Math.ceil(planet.radius / 10));
      for(var a = 0; a < numOfSegments; a++) {
        var angle = a * 2 * (Math.PI / numOfSegments);
        c.beginPath();
        c.moveTo(planet.x + ( 7 * Math.sin(angle)) + ((radius + 24) * Math.cos(angle)), planet.y + ( 7 * Math.cos(angle)) - ((radius + 24) * Math.sin(angle)));
        c.lineTo(planet.x + (-7 * Math.sin(angle)) + ((radius + 24) * Math.cos(angle)), planet.y + (-7 * Math.cos(angle)) - ((radius + 24) * Math.sin(angle)));
        c.lineTo(planet.x                           + ((radius +  8) * Math.cos(angle)), planet.y                           - ((radius +  8) * Math.sin(angle)));
        c.closePath();
        c.fillStyle = this.LEVEL_COLORS[this.currentQuestLevel];
        c.fill();
      }
      c.globalAlpha = 1.0;
      if(this.approach2Animation >= 0.3) {
        this.approach2Planet = -1;
      }
    }
    
  };
  
  
  this.drawQuestArrow = function() {
    if(this.hasQuest) {
      var planet = map.planets[this.currentQuest];
      if(planet.distanceToPlayer > (500 / player.baseZoom)) {
        var angleToQuest = Math.atan2(planet.y - player.y, planet.x - player.x) - (Math.PI / 2);
        angleToQuest += player.cameraAngle;
        c.globalAlpha = 0.5;
        c.save();
        c.translate(player.POS_IN_SCREEN_X, player.POS_IN_SCREEN_Y);
        c.rotate(angleToQuest);
        
        var distToPlayer = 300;
        if(Math.cos(angleToQuest) * distToPlayer > 90) {
          distToPlayer *= 90 / (Math.cos(angleToQuest) * distToPlayer);
        }
        c.translate(0, distToPlayer);
        
        c.beginPath();
        c.moveTo(0, 40);
        c.lineTo(18, 0);
        c.lineTo(0, 8);
        c.lineTo(-18, 0);
        c.closePath();
        c.fillStyle = this.LEVEL_COLORS[this.currentQuestLevel];
        c.fill();
        
        c.restore();
        c.globalAlpha = 1.0;
      }
    }
  };
  
}